﻿using Microsoft.AspNetCore.Authentication;
using Microsoft.IdentityModel.Tokens;

using SocialPub.Services;

using System.Text;

namespace SocialPub.Extensions
{
	public static class AddAuthExtension
	{
		public static AuthenticationBuilder AddSocialPubAuth(this AuthenticationBuilder builder, IConfiguration configuration)
		{
			builder.AddJwtBearer(options => {
#if DEBUG
				options.RequireHttpsMetadata = false;
#endif
				options.TokenValidationParameters = new()
				{
					ValidateIssuer = true,
					ValidateAudience = true,
					ValidateLifetime = true,
					ValidateIssuerSigningKey = true,
					ValidIssuer = configuration["AppConfiguration:Jwt:Issuer"],
					ValidAudience = configuration["AppConfiguration:Jwt:Audience"],
					IssuerSigningKey = new SymmetricSecurityKey(Encoding.UTF8.GetBytes(configuration["AppConfiguration:Jwt:Key"]))
				};
				options.Events = new JwtEvents();
			});
			return builder;
		}
	}
}