﻿using Microsoft.AspNetCore.Authentication.JwtBearer;
using Microsoft.AspNetCore.ResponseCompression;

using SocialPub.ClientModels;
using SocialPub.Extensions;
using SocialPub.Models;
using SocialPub.Services;
using SocialPub.StaticServices;
using NSign.AspNetCore;

using System.Text.Json.Serialization;
using NSign.Providers;
using NSign;
using System.Security.Cryptography.X509Certificates;
using NSign.Signatures;
using System.Net.Http.Headers;
using Microsoft.Extensions.DependencyInjection;
using System.Security.Claims;
using Microsoft.Extensions.Caching.Memory;
using NSign.Client;
using static NSign.Constants;
using System.Text;
using Microsoft.OpenApi.Models;
using SocialPub.Services.ClientToServer.Public;

namespace SocialPub.Middleware
{
    public static class SocialPubConfigurations
	{
		public static IServiceCollection socialPubAppSettingsConfiguration(this IServiceCollection service, IConfiguration configuration)
		{
			return service
				.Configure<MongoSettings>(configuration.GetSection(nameof(MongoSettings)))
				.Configure<AppConfiguration>(configuration.GetSection(nameof(AppConfiguration)));
		}
		public static IServiceCollection socialPubWorkersConfiguration(this IServiceCollection service)
		{
			return service;
			//.AddHostedService<DiscussionsWorker>()
			//.AddHostedService<GroupsCleanerWorker>()
			//.AddHostedService<PoliciesCleanerWorker>();
		}
		public static IServiceCollection socialPubHTTPSignature(this IServiceCollection service, IConfiguration configuration)
		{
			//HTTP CLIENT
			service.Configure<AddDigestOptions>(options => options.WithHash(AddDigestOptions.Hash.Sha256))
				.ConfigureMessageSigningOptions(options =>
				{
					options.SignatureName = "SocialPub";
					options
							.WithMandatoryComponent(SignatureComponent.Path)
							.WithMandatoryComponent(SignatureComponent.RequestTarget)
							.SetParameters = signatureParams => signatureParams.WithKeyId("keyId");
				})
				.Services.Configure<SignatureVerificationOptions>(options =>
				{

				})
				.AddHttpClient<ActivityPubClient>(nameof(ActivityPubClient))
				.ConfigureHttpClient(httpClient =>
				{
					httpClient.DefaultRequestHeaders.Accept.Add(new("application/ld+json"));
				})
				.AddDigestAndSigningHandlers()
				//.AddSignatureVerifiationHandler()
				.Services
				.AddSingleton<ISigner>(new HmacSha256SignatureProvider(Encoding.UTF8.GetBytes(configuration["AppConfiguration:Jwt:Key"])));

			//MESSAGE RESPONSE


			return service;
			//.Configure<RequestSignatureVerificationOptions>(options =>
			//{
			//	options.SignaturesToVerify.Add("sample");
			//	options.RequiredSignatureComponents.Add(SignatureComponent.Path);
			//	options.RequiredSignatureComponents.Add(SignatureComponent.Method);
			//	options.RequiredSignatureComponents.Add(SignatureComponent.Digest);
			//})
			//.AddSignatureVerification(serviceProvider =>
			//{
			//	var memoryCache = serviceProvider.GetRequiredService<IMemoryCache>();
			//	//var httpContextAccessor = serviceProvider.GetRequiredService<IHttpContextAccessor>();

			//	//httpContextAccessor.HttpContext.Request.

			//	var cert = memoryCache.GetOrCreate("SocialPub", (cacheEntry) => Extensions.Extensions.GetX509Certificate2("socialPubCert"));
			//	return new RsaPkcs15Sha256SignatureProvider(cert, "anon");
			//})
			//.ConfigureMessageSigningOptions(options =>
			//{
			//	options.WithMandatoryComponent(SignatureComponent.Path)
			//		.WithMandatoryComponent(SignatureComponent.Method)
			//		.WithMandatoryComponent(SignatureComponent.Digest)
			//		.WithOptionalComponent(new HttpHeaderDictionaryStructuredComponent(NSign.Constants.Headers.Signature, "sample", bindRequest: true));
			//	options.SignatureName = "resp";
			//	options.SetParameters = (sigParams) =>
			//	{
			//		sigParams.WithCreatedNow();
			//	};
			//})
			//.ValidateOnStart()
			//.Services
			//.AddHttpClient("ActivityPub", (serviceProvider, client) =>
			//{
			//	client.DefaultRequestHeaders.UserAgent.Add(new ProductInfoHeaderValue("NSignSample", "0.1-beta"));
			//}).Services;
			//.AddSingleton<ISigner>(new RsaPssSha512SignatureProvider(
			//						new X509Certificate2(@"path\to\certificate.pfx", "PasswordForPfx"),
			//						"my-cert"));
		}
		public static IServiceCollection socialPubAuthServicesConfiguration(this IServiceCollection service, IConfiguration configuration)
		{
			return service
				.AddAuthorization(options =>
				{
					options.AddPolicy(Policies.IsUser, Extensions.Extensions.IsUserPolicy());
					options.AddPolicy(Policies.IsAdmin, Extensions.Extensions.IsAdminPolicy());
					options.AddPolicy(Policies.IsModerator, Extensions.Extensions.IsModeratorPolicy());
				})
				.AddAuthentication(options =>
				{
					options.DefaultAuthenticateScheme = JwtBearerDefaults.AuthenticationScheme;
					options.DefaultChallengeScheme = JwtBearerDefaults.AuthenticationScheme;
					options.DefaultScheme = JwtBearerDefaults.AuthenticationScheme;
				})
				.AddSocialPubAuth(configuration)
				.Services
				.AddSingleton<AuthTokenManager>()
				.AddSingleton<IPasswordHasher, PasswordHasher>();
		}
		public static IServiceCollection socialPubInternalizationConfiguration(this IServiceCollection service, IConfiguration configuration)
		{
			return service
				.AddLocalization()
				.AddSingleton<RequestLocalizationOptionsService>();
		}

		public static IServiceCollection socialPubOptimizationConfiguration(this IServiceCollection service)
		{
			return service.AddResponseCompression(opts =>
			{
				opts.Providers.Add<BrotliCompressionProvider>();
				opts.MimeTypes = ResponseCompressionDefaults.MimeTypes.Concat(new[] { "application/octet-stream" });
			});
		}

		public static IServiceCollection socialPubDataBaseConfiguration(this IServiceCollection service)
		{
			return service.AddSingleton<DbEntities>();
		}

		public static IServiceCollection socialPubServicesConfiguration(this IServiceCollection service)
		{
			return service
				.AddTransient<IDataService, DataService>()
				.AddTransient<IRootUsersService, RootUsersService>()
				.AddTransient<IGroupUsersService, GroupUsersService>()
				.AddSingleton<AppConfigurationService>()
				.AddHttpContextAccessor()
				.AddMemoryCache()
				.AddSingleton<IPasswordHasher, PasswordHasher>();
		}

		public static IServiceCollection socialPubMiddlewareConfiguration(this IServiceCollection service)
		{
			return service
				.AddEndpointsApiExplorer()
				.AddSwaggerGen(c =>
				{
					c.AddSecurityDefinition("Bearer", new()
					{
						In = ParameterLocation.Header,
						Description = "Please enter a valid token",
						Name = "Authorization",
						Type = SecuritySchemeType.Http,
						BearerFormat = "JWT",
						Scheme = "Bearer"
					});
					c.AddSecurityRequirement(new()
					{
						{
							new()
							{
								Reference = new()
								{
									Type = ReferenceType.SecurityScheme,
									Id = "Bearer"
								}
							},
							new string[]{}
						}
					});
				})
				.AddControllers(options => { options.Filters.Add<OperationCancelledExceptionFilter>(); })
				.AddJsonOptions(options =>
				{
					options.JsonSerializerOptions.IgnoreReadOnlyFields = false;
					options.JsonSerializerOptions.IgnoreReadOnlyProperties = false;
					options.JsonSerializerOptions.PropertyNameCaseInsensitive = true;
					options.JsonSerializerOptions.DefaultIgnoreCondition = JsonIgnoreCondition.WhenWritingNull;
					options.JsonSerializerOptions.Converters.Add(new JsonStringEnumConverter());
				}).Services;
		}

		public static IServiceCollection socialPubCORSConfiguration(this IServiceCollection service)
		{
			return service.AddCors(options =>
			{
				options.DefaultPolicyName = "DefaultCORS";
				options.AddDefaultPolicy(configure =>
				{
					configure.AllowAnyMethod()
						.AllowAnyHeader()
						.AllowAnyOrigin()
						.AllowAnyMethod()
						.DisallowCredentials();
				});
			});
		}

	}
}
